import { InjectionKey } from 'vue'
import { createStore, Store, createLogger } from 'vuex'
import { LOGIN_SUCCESS, LOGOUT, TITLE } from './mutation-types'
import { FETCH_ACCOUNT, FETCH_BUDGET, GET, REGISTER, IMPORT_YNAB, LOGIN, NEW_BUDGET, POST, SET_CURRENT_ACCOUNT, SET_CURRENT_BUDGET } from './action-types'
import { budgetStore } from './budget'
import { Budget, useSessionStore } from '../stores/session'

export const key: InjectionKey<Store<State>> = Symbol()

export const store = createStore<State>({
    state: {
        ShowMenu: undefined,
        CurrentBudgetID: undefined,
    },
    mutations: {
        [TITLE](state, title) {
            document.title = "Budgeteer - " + title;
        },
    },
    actions: {
        [IMPORT_YNAB]({ getters, dispatch }, formData) {
            return dispatch("POST", { path: "/budget/" + getters.CurrentBudget.ID + "/import/ynab", body: formData });
        },
        [GET]({ getters }, { path }) {
            return fetch("/api/v1" + path, {
                headers: getters.AuthHeaders,
            })
        },
        [POST]({ getters }, { path, body }) {
            return fetch("/api/v1" + path, {
                method: "POST",
                headers: getters.AuthHeaders,
                body: body,
            })
        },
        /*async fetchDashboard ({state, commit, rootState}) {
            const response = await fetch("/api/v1/dashboard", {
                headers: {
                    'Authorization': 'Bearer ' + rootState.Session.Token
                }
            })
            const data = await response.json();
            commit("setBudgets", data.Budgets);
        },*/
        async [NEW_BUDGET]({ state, commit, dispatch, rootState }, budgetName) {
            const result = await dispatch("POST", {
                path: "/budget/new",
                body: JSON.stringify({ name: budgetName })
            });
            const response = await result.json();
            commit("addBudget", response)
        },
        async [SET_CURRENT_BUDGET]({ state, commit, dispatch, rootState }, budgetid) {
            commit("setCurrentBudgetID", budgetid);

            if (budgetid == null)
                return

            await dispatch(FETCH_BUDGET, budgetid)
        },
    },
    getters: {
        AuthHeaders(state) {
            return {
                'Authorization': 'Bearer ' + state.Session.Token
            }
        },
        CurrentBudget(state) : Budget | undefined {
            if (state.CurrentBudgetID == null)
                return undefined;

            return state.Budgets.get(state.CurrentBudgetID);
        },
        CurrentBudgetID(state) : string | undefined {
            return state.CurrentBudgetID;
        },
        CurrentBudgetName(state) : string {
            if (state.CurrentBudgetID == null)
                return "";

            const currentBudget = state.Budgets.get(state.CurrentBudgetID);
            if(currentBudget != undefined)
                return currentBudget.Name;
            return "";
        },
    },
    plugins: [createLogger()],
    modules: {
        budget: budgetStore
    }
})

store.subscribe((mutation, state) => {

    const sessionStore = useSessionStore();
    let persistedState = {
        Session: sessionStore,
//        Accounts: [...state.Accounts],
        CurrentBudgetID: state.CurrentBudgetID,
        //CurrentAccountID: state.CurrentAccountID,
        ExpandMenu: state.ExpandMenu,
        ShowMenu: state.ShowMenu
    }
    localStorage.setItem("store", JSON.stringify(persistedState));
})